# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ModelListResponse", "Data"]


class Data(BaseModel):
    id: str
    """The unique identifier of the model.

    This can be used to specify the model in API requests. For fine-tuned models,
    this may include a user-specific prefix.
    """

    created: int
    """Unix timestamp (in seconds) when this model was created or made available.

    For fine-tuned models, this represents when the fine-tuning was completed.
    """

    object: Literal["model"]
    """The type of object represented, which is always "model" for model objects.

    This helps distinguish model objects from other types of responses.
    """

    owned_by: str
    """Identifies the owner or provider of the model.

    Can be "premai" for base models, a user ID for fine-tuned models, or other
    providers like "openai" or "anthropic".
    """


class ModelListResponse(BaseModel):
    data: List[Data]
    """An array containing the available models.

    Each element is a complete model object with all its properties.
    """

    object: Literal["list"]
    """The type of object returned, always "list" for model listing responses.

    Helps identify this as a collection of models.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["ModelCheckStatusResponse"]


class ModelCheckStatusResponse(BaseModel):
    status: bool
    """Whether the model is running."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ChatCompletionsParams", "Message", "MessageToolCall", "MessageToolCallFunction", "ResponseFormat"]


class ChatCompletionsParams(TypedDict, total=False):
    messages: Required[Iterable[Message]]
    """An array of messages comprising the conversation so far.

    Must contain at least one message. System messages are only allowed as the first
    message.
    """

    model: Required[str]
    """The identifier of the model to use for generating completions.

    This can be a model ID or an alias.
    """

    frequency_penalty: float
    """
    A value between -2.0 and 2.0 that penalizes new tokens based on their frequency
    in the text so far. Higher values decrease the likelihood of the model repeating
    the same tokens.
    """

    max_completion_tokens: Optional[int]
    """The maximum number of tokens to generate in the completion.

    If null, will use the model's maximum context length. This is the maximum number
    of tokens that will be generated.
    """

    presence_penalty: float
    """
    A value between -2.0 and 2.0 that penalizes new tokens based on whether they
    appear in the text so far. Higher values increase the likelihood of the model
    talking about new topics.
    """

    response_format: ResponseFormat
    """Specifies the format of the model's output.

    Use "json_schema" to constrain responses to valid JSON matching the provided
    schema.
    """

    seed: int
    """A seed value for deterministic sampling.

    Using the same seed with the same parameters will generate the same completion.
    """

    stop: Union[str, List[str]]
    """One or more sequences where the API will stop generating further tokens.

    Can be a single string or an array of strings.
    """

    stream: bool
    """If true, partial message deltas will be sent as server-sent events.

    Useful for showing progressive generation in real-time.
    """

    temperature: Optional[float]
    """Controls randomness in the model's output.

    Values between 0 and 2. Lower values make the output more focused and
    deterministic, higher values make it more random and creative.
    """

    tool_choice: Union[Literal["none", "auto"], object]
    """Controls how the model uses tools.

    "none" disables tools, "auto" lets the model decide, or specify a particular
    tool configuration.
    """

    tools: Iterable[object]
    """A list of tools the model may call.

    Each tool has a specific function the model can use to achieve specific tasks.
    """

    top_p: Optional[float]
    """An alternative to temperature for controlling randomness.

    Controls the cumulative probability of tokens to consider. Lower values make
    output more focused.
    """


class MessageToolCallFunction(TypedDict, total=False):
    arguments: Required[str]
    """The arguments to pass to the function as a JSON string"""

    name: Required[str]
    """The name of the function to call"""


class MessageToolCall(TypedDict, total=False):
    id: Required[str]
    """A unique identifier for this tool call"""

    function: Required[MessageToolCallFunction]

    type: Required[Literal["function"]]
    """The type of tool call"""


class Message(TypedDict, total=False):
    role: Required[Literal["system", "user", "assistant", "tool"]]
    """The role of the message sender.

    "system" is for system-level instructions, "user" represents the end user, and
    "assistant" represents the AI model's responses.
    """

    content: Optional[str]
    """The actual text content of the message.

    Note that this can be null in certain cases, such as when the message contains
    tool calls or when specific roles don't require content.
    """

    name: str
    """The name of the function to call, if any."""

    tool_call_id: str
    """
    The ID of the tool call that this message is a response to (only for tool role
    messages)
    """

    tool_calls: Iterable[MessageToolCall]
    """Tool calls to be made by the assistant"""


class ResponseFormat(TypedDict, total=False):
    json_schema: Required[Dict[str, object]]

    type: Required[Literal["text", "json_schema"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ModelLoadParams"]


class ModelLoadParams(TypedDict, total=False):
    model: Required[str]
    """The ID or alias of the model to load up."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Literal

import httpx

from ..types import chat_completions_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.chat_completions_response import ChatCompletionsResponse

__all__ = ["ChatResource", "AsyncChatResource"]


class ChatResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return ChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return ChatResourceWithStreamingResponse(self)

    def completions(
        self,
        *,
        messages: Iterable[chat_completions_params.Message],
        model: str,
        frequency_penalty: float | NotGiven = NOT_GIVEN,
        max_completion_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        presence_penalty: float | NotGiven = NOT_GIVEN,
        response_format: chat_completions_params.ResponseFormat | NotGiven = NOT_GIVEN,
        seed: int | NotGiven = NOT_GIVEN,
        stop: Union[str, List[str]] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        tool_choice: Union[Literal["none", "auto"], object] | NotGiven = NOT_GIVEN,
        tools: Iterable[object] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCompletionsResponse:
        """
        Create a chat completion (OpenAI compatible).

        Args:
          messages: An array of messages comprising the conversation so far. Must contain at least
              one message. System messages are only allowed as the first message.

          model: The identifier of the model to use for generating completions. This can be a
              model ID or an alias.

          frequency_penalty: A value between -2.0 and 2.0 that penalizes new tokens based on their frequency
              in the text so far. Higher values decrease the likelihood of the model repeating
              the same tokens.

          max_completion_tokens: The maximum number of tokens to generate in the completion. If null, will use
              the model's maximum context length. This is the maximum number of tokens that
              will be generated.

          presence_penalty: A value between -2.0 and 2.0 that penalizes new tokens based on whether they
              appear in the text so far. Higher values increase the likelihood of the model
              talking about new topics.

          response_format: Specifies the format of the model's output. Use "json_schema" to constrain
              responses to valid JSON matching the provided schema.

          seed: A seed value for deterministic sampling. Using the same seed with the same
              parameters will generate the same completion.

          stop: One or more sequences where the API will stop generating further tokens. Can be
              a single string or an array of strings.

          stream: If true, partial message deltas will be sent as server-sent events. Useful for
              showing progressive generation in real-time.

          temperature: Controls randomness in the model's output. Values between 0 and 2. Lower values
              make the output more focused and deterministic, higher values make it more
              random and creative.

          tool_choice: Controls how the model uses tools. "none" disables tools, "auto" lets the model
              decide, or specify a particular tool configuration.

          tools: A list of tools the model may call. Each tool has a specific function the model
              can use to achieve specific tasks.

          top_p: An alternative to temperature for controlling randomness. Controls the
              cumulative probability of tokens to consider. Lower values make output more
              focused.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/chat/completions",
            body=maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "frequency_penalty": frequency_penalty,
                    "max_completion_tokens": max_completion_tokens,
                    "presence_penalty": presence_penalty,
                    "response_format": response_format,
                    "seed": seed,
                    "stop": stop,
                    "stream": stream,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_p": top_p,
                },
                chat_completions_params.ChatCompletionsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCompletionsResponse,
        )


class AsyncChatResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChatResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncChatResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChatResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return AsyncChatResourceWithStreamingResponse(self)

    async def completions(
        self,
        *,
        messages: Iterable[chat_completions_params.Message],
        model: str,
        frequency_penalty: float | NotGiven = NOT_GIVEN,
        max_completion_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        presence_penalty: float | NotGiven = NOT_GIVEN,
        response_format: chat_completions_params.ResponseFormat | NotGiven = NOT_GIVEN,
        seed: int | NotGiven = NOT_GIVEN,
        stop: Union[str, List[str]] | NotGiven = NOT_GIVEN,
        stream: bool | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        tool_choice: Union[Literal["none", "auto"], object] | NotGiven = NOT_GIVEN,
        tools: Iterable[object] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCompletionsResponse:
        """
        Create a chat completion (OpenAI compatible).

        Args:
          messages: An array of messages comprising the conversation so far. Must contain at least
              one message. System messages are only allowed as the first message.

          model: The identifier of the model to use for generating completions. This can be a
              model ID or an alias.

          frequency_penalty: A value between -2.0 and 2.0 that penalizes new tokens based on their frequency
              in the text so far. Higher values decrease the likelihood of the model repeating
              the same tokens.

          max_completion_tokens: The maximum number of tokens to generate in the completion. If null, will use
              the model's maximum context length. This is the maximum number of tokens that
              will be generated.

          presence_penalty: A value between -2.0 and 2.0 that penalizes new tokens based on whether they
              appear in the text so far. Higher values increase the likelihood of the model
              talking about new topics.

          response_format: Specifies the format of the model's output. Use "json_schema" to constrain
              responses to valid JSON matching the provided schema.

          seed: A seed value for deterministic sampling. Using the same seed with the same
              parameters will generate the same completion.

          stop: One or more sequences where the API will stop generating further tokens. Can be
              a single string or an array of strings.

          stream: If true, partial message deltas will be sent as server-sent events. Useful for
              showing progressive generation in real-time.

          temperature: Controls randomness in the model's output. Values between 0 and 2. Lower values
              make the output more focused and deterministic, higher values make it more
              random and creative.

          tool_choice: Controls how the model uses tools. "none" disables tools, "auto" lets the model
              decide, or specify a particular tool configuration.

          tools: A list of tools the model may call. Each tool has a specific function the model
              can use to achieve specific tasks.

          top_p: An alternative to temperature for controlling randomness. Controls the
              cumulative probability of tokens to consider. Lower values make output more
              focused.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/chat/completions",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "frequency_penalty": frequency_penalty,
                    "max_completion_tokens": max_completion_tokens,
                    "presence_penalty": presence_penalty,
                    "response_format": response_format,
                    "seed": seed,
                    "stop": stop,
                    "stream": stream,
                    "temperature": temperature,
                    "tool_choice": tool_choice,
                    "tools": tools,
                    "top_p": top_p,
                },
                chat_completions_params.ChatCompletionsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChatCompletionsResponse,
        )


class ChatResourceWithRawResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

        self.completions = to_raw_response_wrapper(
            chat.completions,
        )


class AsyncChatResourceWithRawResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

        self.completions = async_to_raw_response_wrapper(
            chat.completions,
        )


class ChatResourceWithStreamingResponse:
    def __init__(self, chat: ChatResource) -> None:
        self._chat = chat

        self.completions = to_streamed_response_wrapper(
            chat.completions,
        )


class AsyncChatResourceWithStreamingResponse:
    def __init__(self, chat: AsyncChatResource) -> None:
        self._chat = chat

        self.completions = async_to_streamed_response_wrapper(
            chat.completions,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from premai import PremAI, AsyncPremAI
from tests.utils import assert_matches_type
from premai.types import ChatCompletionsResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChat:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_completions(self, client: PremAI) -> None:
        chat = client.chat.completions(
            messages=[{"role": "system"}],
            model="model",
        )
        assert_matches_type(ChatCompletionsResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_completions_with_all_params(self, client: PremAI) -> None:
        chat = client.chat.completions(
            messages=[
                {
                    "role": "system",
                    "content": "content",
                    "name": "name",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "id": "id",
                            "function": {
                                "arguments": "arguments",
                                "name": "name",
                            },
                            "type": "function",
                        }
                    ],
                }
            ],
            model="model",
            frequency_penalty=-2,
            max_completion_tokens=1,
            presence_penalty=-2,
            response_format={
                "json_schema": {"foo": "bar"},
                "type": "text",
            },
            seed=0,
            stop="string",
            stream=True,
            temperature=0,
            tool_choice="none",
            tools=[{}],
            top_p=0,
        )
        assert_matches_type(ChatCompletionsResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_completions(self, client: PremAI) -> None:
        response = client.chat.with_raw_response.completions(
            messages=[{"role": "system"}],
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatCompletionsResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_completions(self, client: PremAI) -> None:
        with client.chat.with_streaming_response.completions(
            messages=[{"role": "system"}],
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatCompletionsResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncChat:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_completions(self, async_client: AsyncPremAI) -> None:
        chat = await async_client.chat.completions(
            messages=[{"role": "system"}],
            model="model",
        )
        assert_matches_type(ChatCompletionsResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_completions_with_all_params(self, async_client: AsyncPremAI) -> None:
        chat = await async_client.chat.completions(
            messages=[
                {
                    "role": "system",
                    "content": "content",
                    "name": "name",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "id": "id",
                            "function": {
                                "arguments": "arguments",
                                "name": "name",
                            },
                            "type": "function",
                        }
                    ],
                }
            ],
            model="model",
            frequency_penalty=-2,
            max_completion_tokens=1,
            presence_penalty=-2,
            response_format={
                "json_schema": {"foo": "bar"},
                "type": "text",
            },
            seed=0,
            stop="string",
            stream=True,
            temperature=0,
            tool_choice="none",
            tools=[{}],
            top_p=0,
        )
        assert_matches_type(ChatCompletionsResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_completions(self, async_client: AsyncPremAI) -> None:
        response = await async_client.chat.with_raw_response.completions(
            messages=[{"role": "system"}],
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatCompletionsResponse, chat, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_completions(self, async_client: AsyncPremAI) -> None:
        async with async_client.chat.with_streaming_response.completions(
            messages=[{"role": "system"}],
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatCompletionsResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .model_load_params import ModelLoadParams as ModelLoadParams
from .model_list_response import ModelListResponse as ModelListResponse
from .model_unload_params import ModelUnloadParams as ModelUnloadParams
from .chat_completions_params import ChatCompletionsParams as ChatCompletionsParams
from .chat_completions_response import ChatCompletionsResponse as ChatCompletionsResponse
from .model_check_status_params import ModelCheckStatusParams as ModelCheckStatusParams
from .model_check_status_response import ModelCheckStatusResponse as ModelCheckStatusResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import model_load_params, model_unload_params, model_check_status_params
from .._types import NOT_GIVEN, Body, Query, Headers, NoneType, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.model_list_response import ModelListResponse
from ..types.model_check_status_response import ModelCheckStatusResponse

__all__ = ["ModelsResource", "AsyncModelsResource"]


class ModelsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return ModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return ModelsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelListResponse:
        """Get available AI models in OpenAI compatible format."""
        return self._get(
            "/api/v1/models",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelListResponse,
        )

    def check_status(
        self,
        *,
        model: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelCheckStatusResponse:
        """Check if a model is running for inference.

        Use query param to support IDs with /

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/api/v1/models/running",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"model": model}, model_check_status_params.ModelCheckStatusParams),
            ),
            cast_to=ModelCheckStatusResponse,
        )

    def load(
        self,
        *,
        model: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """Load up a model for inference.

        This endpoint requests a model to be loaded into
        memory for faster inference.

        Args:
          model: The ID or alias of the model to load up.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/api/v1/models/up",
            body=maybe_transform({"model": model}, model_load_params.ModelLoadParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def unload(
        self,
        *,
        model: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Load down a model for inference.

        Args:
          model: The ID or alias of the model to load down.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/api/v1/models/down",
            body=maybe_transform({"model": model}, model_unload_params.ModelUnloadParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncModelsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncModelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncModelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/premAI-io/prem-py-sdk#with_streaming_response
        """
        return AsyncModelsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelListResponse:
        """Get available AI models in OpenAI compatible format."""
        return await self._get(
            "/api/v1/models",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ModelListResponse,
        )

    async def check_status(
        self,
        *,
        model: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ModelCheckStatusResponse:
        """Check if a model is running for inference.

        Use query param to support IDs with /

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/api/v1/models/running",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"model": model}, model_check_status_params.ModelCheckStatusParams),
            ),
            cast_to=ModelCheckStatusResponse,
        )

    async def load(
        self,
        *,
        model: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """Load up a model for inference.

        This endpoint requests a model to be loaded into
        memory for faster inference.

        Args:
          model: The ID or alias of the model to load up.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/api/v1/models/up",
            body=await async_maybe_transform({"model": model}, model_load_params.ModelLoadParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def unload(
        self,
        *,
        model: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> None:
        """
        Load down a model for inference.

        Args:
          model: The ID or alias of the model to load down.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/api/v1/models/down",
            body=await async_maybe_transform({"model": model}, model_unload_params.ModelUnloadParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ModelsResourceWithRawResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

        self.list = to_raw_response_wrapper(
            models.list,
        )
        self.check_status = to_raw_response_wrapper(
            models.check_status,
        )
        self.load = to_raw_response_wrapper(
            models.load,
        )
        self.unload = to_raw_response_wrapper(
            models.unload,
        )


class AsyncModelsResourceWithRawResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

        self.list = async_to_raw_response_wrapper(
            models.list,
        )
        self.check_status = async_to_raw_response_wrapper(
            models.check_status,
        )
        self.load = async_to_raw_response_wrapper(
            models.load,
        )
        self.unload = async_to_raw_response_wrapper(
            models.unload,
        )


class ModelsResourceWithStreamingResponse:
    def __init__(self, models: ModelsResource) -> None:
        self._models = models

        self.list = to_streamed_response_wrapper(
            models.list,
        )
        self.check_status = to_streamed_response_wrapper(
            models.check_status,
        )
        self.load = to_streamed_response_wrapper(
            models.load,
        )
        self.unload = to_streamed_response_wrapper(
            models.unload,
        )


class AsyncModelsResourceWithStreamingResponse:
    def __init__(self, models: AsyncModelsResource) -> None:
        self._models = models

        self.list = async_to_streamed_response_wrapper(
            models.list,
        )
        self.check_status = async_to_streamed_response_wrapper(
            models.check_status,
        )
        self.load = async_to_streamed_response_wrapper(
            models.load,
        )
        self.unload = async_to_streamed_response_wrapper(
            models.unload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "ChatCompletionsResponse",
    "Choice",
    "ChoiceMessage",
    "ChoiceMessageToolCall",
    "ChoiceMessageToolCallFunction",
    "Usage",
]


class ChoiceMessageToolCallFunction(BaseModel):
    arguments: str

    name: str


class ChoiceMessageToolCall(BaseModel):
    id: str

    function: ChoiceMessageToolCallFunction

    type: Literal["function"]


class ChoiceMessage(BaseModel):
    content: Optional[str] = None

    role: Literal["assistant", "tool"]

    name: Optional[str] = None

    tool_call_id: Optional[str] = None

    tool_calls: Optional[List[ChoiceMessageToolCall]] = None


class Choice(BaseModel):
    finish_reason: Literal["stop", "length", "tool_calls", "content_filter", "function_call"]

    index: int

    message: ChoiceMessage


class Usage(BaseModel):
    completion_tokens: int
    """The number of tokens generated in the completion response.

    This counts only the tokens in the assistant's reply.
    """

    prompt_tokens: int
    """The number of tokens in the prompt/input.

    This includes all messages sent to the API in the conversation history.
    """

    total_tokens: int


class ChatCompletionsResponse(BaseModel):
    id: str
    """A unique identifier for this chat completion response.

    Can be used for tracking or debugging.
    """

    choices: List[Choice]
    """An array of completion choices.

    Each choice represents a possible completion for the input prompt, though
    currently only one choice is typically returned.
    """

    created: int
    """
    The Unix timestamp (in seconds) indicating when this completion was generated by
    the API.
    """

    model: str
    """The specific model used to generate this completion.

    This will be the model's full identifier string.
    """

    object: Literal["chat.completion"]
    """
    The type of object returned, always "chat.completion" for chat completion
    responses.
    """

    system_fingerprint: Optional[str] = None
    """A unique identifier for the system state that generated this response.

    Useful for tracking model behavior across requests.
    """

    usage: Optional[Usage] = None
    """Statistics about token usage for this request and response.

    May be omitted in error cases or when not available.
    """
